@@ -9,7 +9,6 @@ use syndicate::language;
 use syndicate::actor::*;
 use syndicate::during::entity;
 use syndicate::dataspace::Dataspace;
-use syndicate::enclose;
 use syndicate::schemas::dataspace::Observe;
 use syndicate::schemas::dataspace_patterns as p;
 use syndicate::value::NestedValue;
@@ -52,24 +51,18 @@ pub fn bench_pub(c: &mut Criterion) {
         b.iter_custom(|iters| {
             let start = Instant::now();
             rt.block_on(async move {
-                Actor::new(None).boot(syndicate::name!("dataspace"), move |t| {
+                Actor::top(None, move |t| {
+                    let _ = t.prevent_inert_check();
+                    // The reason this works is that all the messages to `ds` will be delivered
+                    // before the message to `shutdown`, because `ds` and `shutdown` are in the
+                    // same Actor.
                     let ds = t.create(Dataspace::new(None));
                     let shutdown = t.create(ShutdownEntity);
-                    let account = Account::new(syndicate::name!("sender-account"));
-                    t.linked_task(syndicate::name!("sender"), async move {
-                        for _ in 0..iters {
-                            external_event(&ds.mailbox, &account, Box::new(
-                                enclose!((ds) move |t| t.with_entity(
-                                    &ds,
-                                    |t, e| e.message(t, says(AnyValue::new("bench_pub"),
-                                                             Value::ByteString(vec![]).wrap()))))))?
-                        }
-                        external_event(&shutdown.mailbox, &account, Box::new(
-                            enclose!((shutdown) move |t| t.with_entity(
-                                &shutdown,
-                                |t, e| e.message(t, AnyValue::new(true))))))?;
-                        Ok(LinkedTaskTermination::KeepFacet)
-                    });
+                    for _ in 0..iters {
+                        t.message(&ds, says(AnyValue::new("bench_pub"),
+                                            Value::ByteString(vec![]).wrap()));
+                    }
+                    t.message(&shutdown, AnyValue::new(true));
                     Ok(())
                 }).await.unwrap().unwrap();
             });
@@ -83,7 +76,7 @@ pub fn bench_pub(c: &mut Criterion) {
             rt.block_on(async move {
                 let turn_count = Arc::new(AtomicU64::new(0));
 
-                Actor::new(None).boot(syndicate::name!("dataspace"), {
+                Actor::top(None, {
                     let iters = iters.clone();
                     let turn_count = Arc::clone(&turn_count);
 
@@ -103,7 +96,7 @@ pub fn bench_pub(c: &mut Criterion) {
                             observer: shutdown,
                         });
 
-                        t.spawn(syndicate::name!("consumer"), move |t| {
+                        t.spawn(Some(AnyValue::symbol("consumer")), move |t| {
                             struct Receiver(Arc<AtomicU64>);
                             impl Entity<AnyValue> for Receiver {
                                 fn message(&mut self, _t: &mut Activation, _m: AnyValue) -> ActorResult {
@@ -138,25 +131,15 @@ pub fn bench_pub(c: &mut Criterion) {
                                 observer: shutdown,
                             });
 
-                            let account = Arc::clone(t.account());
-                            t.linked_task(syndicate::name!("sender"), async move {
+                            t.after(core::time::Duration::from_secs(0), move |t| {
                                 for _i in 0..iters {
-                                    let ds = Arc::clone(&ds);
-                                    external_event(&Arc::clone(&ds.underlying.mailbox), &account, Box::new(
-                                        move |t| t.with_entity(
-                                            &ds.underlying,
-                                            |t, e| e.message(t, says(AnyValue::new("bench_pub"),
-                                                                     Value::ByteString(vec![]).wrap())))))?
+                                    ds.message(t, &(), &says(AnyValue::new("bench_pub"),
+                                                             Value::ByteString(vec![]).wrap()));
                                 }
-                                {
-                                    let ds = Arc::clone(&ds);
-                                    external_event(&Arc::clone(&ds.underlying.mailbox), &account, Box::new(
-                                        move |t| t.with_entity(
-                                            &ds.underlying,
-                                            |t, e| e.message(t, AnyValue::new(true)))))?;
-                                }
-                                Ok(LinkedTaskTermination::KeepFacet)
+                                ds.message(t, &(), &AnyValue::new(true));
+                                Ok(())
                             });
+
                             Ok(())
                         });
                         Ok(())
