@@ -1,23 +1,19 @@
 //! Extremely simple single-actor supervision. Vastly simplified compared to the available
 //! options in [Erlang/OTP](https://erlang.org/doc/man/supervisor.html).
 
+use preserves::value::NestedValue;
+
 use std::collections::VecDeque;
 use std::sync::Arc;
+use std::sync::Mutex;
 use std::time::Duration;
 
 use tokio::time::Instant;
 
 use crate::actor::*;
-use crate::enclose;
 use crate::schemas::service::State;
 
-pub type Boot = Box<dyn Send + FnMut(&mut Activation) -> ActorResult>;
-
-enum Protocol {
-    SuperviseeStarted, // assertion
-    BootFunction(Boot), // message
-    Retry, // message
-}
+pub type Boot = Arc<Mutex<Box<dyn Send + FnMut(&mut Activation) -> ActorResult>>>;
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum RestartPolicy {
@@ -34,26 +30,19 @@ pub struct SupervisorConfiguration {
     pub restart_policy: RestartPolicy,
 }
 
+#[derive(Debug)]
+struct StartNow;
+
 pub struct Supervisor {
-    self_ref: Arc<Ref<Protocol>>,
-    name: tracing::Span,
+    self_ref: Arc<Ref<StartNow>>,
+    child_name: Name,
     config: SupervisorConfiguration,
-    boot_fn: Option<Boot>,
+    boot_fn: Boot,
     restarts: VecDeque<Instant>,
     state: Arc<Field<State>>,
     ac_ref: Option<ActorRef>,
 }
 
-impl std::fmt::Debug for Protocol {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        match self {
-            Protocol::SuperviseeStarted => write!(f, "Protocol::SuperviseeStarted"),
-            Protocol::BootFunction(_) => write!(f, "Protocol::BootFunction(_)"),
-            Protocol::Retry => write!(f, "Protocol::Retry"),
-        }
-    }
-}
-
 impl Default for SupervisorConfiguration {
     fn default() -> Self {
         Self {
@@ -75,30 +64,23 @@ impl SupervisorConfiguration {
     }
 }
 
-impl Entity<Protocol> for Supervisor
+impl Entity<StartNow> for Supervisor
 {
-    fn assert(&mut self, t: &mut Activation, m: Protocol, _h: Handle) -> ActorResult {
-        match m {
-            Protocol::SuperviseeStarted => t.set(&self.state, State::Started),
-            _ => Err(format!("Unexpected assertion: {:?}", m).as_str())?,
-        }
-        Ok(())
+    fn message(&mut self, t: &mut Activation, _m: StartNow) -> ActorResult {
+        self.start_now(t)
     }
 
-    fn retract(&mut self, t: &mut Activation, _h: Handle) -> ActorResult {
-        let _name = self.name.clone();
-        let _entry = _name.enter();
-        let exit_status =
-            self.ac_ref.take().expect("valid supervisee ActorRef")
-            .exit_status()
-            .expect("supervisee to have terminated");
-        tracing::debug!(?exit_status);
-        match exit_status {
-            Ok(()) if self.config.restart_policy == RestartPolicy::OnErrorOnly => {
+    fn stop(&mut self, t: &mut Activation) -> ActorResult {
+        let _entry = tracing::info_span!("supervisor", name = ?self.child_name).entered();
+        match self.ac_ref.take().expect("valid supervisee ActorRef").exit_status() {
+            None =>
+                tracing::debug!("Supervisor shut down; supervisee will exit soon"),
+            Some(Ok(())) if self.config.restart_policy == RestartPolicy::OnErrorOnly => {
                 tracing::trace!("Not restarting: normal exit, restart_policy is OnErrorOnly");
                 t.set(&self.state, State::Complete);
             },
-            _ => {
+            Some(exit_status) => {
+                tracing::debug!(?exit_status);
                 tracing::trace!("Restarting: restart_policy is Always or exit was abnormal");
                 t.set(&self.state,
                       if exit_status.is_ok() { State::Complete } else { State::Failed });
@@ -122,105 +104,63 @@ impl Entity<Protocol> for Supervisor
                 };
                 t.after(wait_time, move |t| {
                     tracing::trace!("Sending retry trigger");
-                    t.message(&self_ref, Protocol::Retry);
+                    t.message(&self_ref, StartNow);
                     Ok(())
                 });
             },
         }
         Ok(())
     }
-
-    fn message(&mut self, t: &mut Activation, m: Protocol) -> ActorResult {
-        match m {
-            Protocol::BootFunction(b) => {
-                self.boot_fn = Some(b);
-                Ok(())
-            }
-            Protocol::Retry => {
-                self.ensure_started(t)
-            }
-            _ => Ok(())
-        }
-    }
-
-    fn stop(&mut self, _t: &mut Activation) -> ActorResult {
-        let _entry = self.name.enter();
-        tracing::info!(self_ref = ?self.self_ref, "Supervisor terminating");
-        Ok(())
-    }
 }
 
 impl Supervisor {
     pub fn start<C: 'static + Send + FnMut(&mut Activation, State) -> ActorResult,
                  B: 'static + Send + FnMut(&mut Activation) -> ActorResult>(
         t: &mut Activation,
-        name: tracing::Span,
+        name: Name,
         config: SupervisorConfiguration,
         mut state_cb: C,
         boot_fn: B,
     ) -> ActorResult {
-        let _entry = name.enter();
+        let _entry = tracing::info_span!("supervisor", ?name).entered();
         tracing::trace!(?config);
         let self_ref = t.create_inert();
         let state_field = t.named_field("supervisee_state", State::Started);
+        let my_name = name.as_ref().map(
+            |n| preserves::rec![AnyValue::symbol("supervisor"), n.clone()]);
         let mut supervisor = Supervisor {
             self_ref: Arc::clone(&self_ref),
-            name: name.clone(),
+            child_name: name,
             config,
-            boot_fn: Some(Box::new(boot_fn)),
+            boot_fn: Arc::new(Mutex::new(Box::new(boot_fn))),
             restarts: VecDeque::new(),
             state: Arc::clone(&state_field),
             ac_ref: None,
         };
         tracing::info!(self_ref = ?supervisor.self_ref, "Supervisor starting");
-        supervisor.ensure_started(t)?;
-        t.dataflow(enclose!((name) move |t| {
+        supervisor.start_now(t)?;
+        t.dataflow(move |t| {
             let state = t.get(&state_field).clone();
-            {
-                let _entry = name.enter();
-                tracing::debug!(?state);
-            }
+            tracing::debug!(name = ?my_name, ?state);
             state_cb(t, state)
-        }))?;
+        })?;
         self_ref.become_entity(supervisor);
-        t.on_stop_notify(&self_ref);
         Ok(())
     }
 
-    fn ensure_started(&mut self, t: &mut Activation) -> ActorResult {
-        match self.boot_fn.take() {
-            None => {
-                let _entry = self.name.enter();
-                t.set(&self.state, State::Failed);
-                tracing::error!("Cannot restart supervisee, because it panicked at startup")
-            }
-            Some(mut boot_fn) => {
-                let self_ref = Arc::clone(&self.self_ref);
-                t.facet(|t: &mut Activation| {
-                    t.assert(&self.self_ref, Protocol::SuperviseeStarted);
-                    self.ac_ref = Some(t.spawn_link(
-                        crate::name!(parent: &self.name, "supervisee"),
-                        move |t| {
-                            match boot_fn(t) {
-                                Ok(()) => {
-                                    t.message(&self_ref, Protocol::BootFunction(boot_fn));
-                                    Ok(())
-                                }
-                                Err(e) => {
-                                    t.clear();
-                                    t.message(&self_ref, Protocol::BootFunction(boot_fn));
-                                    t.deliver();
-                                    Err(e)
-                                }
-                            }
-                        }));
-                    tracing::debug!(self_ref = ?self.self_ref,
-                                    supervisee = ?self.ac_ref,
-                                    "Supervisee started");
-                    Ok(())
-                })?;
-            }
-        }
+    fn start_now(&mut self, t: &mut Activation) -> ActorResult {
+        let boot_cell = Arc::clone(&self.boot_fn);
+        t.facet(|t: &mut Activation| {
+            t.on_stop_notify(&self.self_ref);
+            self.ac_ref = Some(t.spawn_link(
+                self.child_name.clone(),
+                move |t| boot_cell.lock().expect("Unpoisoned boot_fn mutex")(t)));
+            tracing::debug!(self_ref = ?self.self_ref,
+                            supervisee = ?self.ac_ref,
+                            "Supervisee started");
+            Ok(())
+        })?;
+        t.set(&self.state, State::Started);
         Ok(())
     }
 }
